@@ -2,16 +2,20 @@ from ..core import *
 
 from .artist import (
     Drawer,
+    Printer,
 )
 
 from .analyst import (
     Regressor,
-    Describer
+    Describer,
+    Decompositer,
+    Tester,
 )
 
 from .fetcher import (
     Filer,
-    Databaser,
+    Sqliter,
+    Mysqler,
 )
 
 from .calculator import (
@@ -19,12 +23,17 @@ from .calculator import (
 )
     
 from .processor import (
-    PreProcessor
+    PreProcessor,
+    Converter,
 )
 
 from .backtester import (
     Relocator,
+    BackTrader,
     Strategy,
+    Indicator,
+    Analyzer,
+    Observer,
     OrderTable,
 )
 
